﻿unit U_CustomizeMenu;


/////////////////////////////////////////////////////////////////////////////////
//  U_CustomizeMenu
// Author : Matthieu GIROUX www.liberlog.fr
/////////////////////////////////////////////////////////////////////////////////

{$IFDEF FPC}
{$mode delphi}
{$ENDIF}

{$I ..\DLCompilers.inc}
{$I ..\extends.inc}

interface

uses
  Classes, SysUtils,
{$IFDEF FPC}
  LazFileUtils,
{$ELSE}
  JvExControls,
{$ENDIF}
  Forms, Controls, Graphics, Dialogs,
  {$IFDEF VERSIONS}
    fonctions_version,
  {$ENDIF}
  {$IFDEF VIRTUALTREES}
  VirtualTrees,
  {$ELSE}
  ComCtrls,
  {$ENDIF}
  u_extmenucustomize,
  U_OnFormInfoIni, Menus, Buttons,
  ExtCtrls, StdCtrls;

{$IFDEF VERSIONS}
const
    gVer_F_CustomizeMenu : T_Version = ( Component : 'Fenêtre de F_CustomizeMenu' ;
       			                 FileUnit : 'U_CustomizeMenu' ;
       			                 Owner : 'Matthieu Giroux' ;
       			                 Comment : 'Fenêtre de personnalisation de menu.' ;
      			                 BugsStory : 'Version 1.0.0.0 : TTreeview.' + #13#10 +
                                                     'Version 0.9.0.0 : Adding Customized Menu.' + #13#10;
			                 UnitType : CST_TYPE_UNITE_FICHE ;
			                 Major : 1 ; Minor : 0 ; Release : 0 ; Build : 0 );
{$ENDIF}

type
  TMenuNode = Record
    Name, Title : String;
    ImageIndex : Integer;
  end;

  PCustMenuNode = ^TMenuNode ;

  { TF_CustomizeMenu }
  TF_CustomizeMenu = class(TForm)
    FWClose1: TSpeedButton;
    FWDelete: TSpeedButton;
    FWInsert: TSpeedButton;
    ch_ajouteravant: TCheckbox;
    Panel1: TPanel;
    PanelTrees: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Splitter1:TSplitter;
    {$IFDEF VIRTUALTREES}
    vt_MainMenu: TVirtualStringTree;
    vt_MenuIni: TVirtualStringTree;
    {$ELSE}
    vt_MainMenu: TTreeView;
    vt_MenuIni: TTreeView;
    {$ENDIF}
    OnFormInfoIni: TOnFormInfoIni;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FWClose1Click(Sender: TObject);
    procedure FWDeleteClick(Sender: TObject);
    procedure FWInsertClick(Sender: TObject);
    procedure vt_MainMenuClick(Sender: TObject);
    {$IFDEF VIRTUALTREES}
    procedure vt_MainMenuGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure vt_MainMenuInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure LoadMenuNode ( const ATree : TVirtualStringTree ; const AMenuItem : TMenuItem ; const ParentNode : PVirtualNode ; const aajouter, aSearchValidity : Boolean ); virtual;
    procedure vt_MainMenuGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: {$IFDEF FPC}String{$ELSE}WideString{$ENDIF});
    {$ELSE}
    procedure vt_MainMenuGetText(Node: TTreeNode);
    procedure vt_MainMenuGetImageIndex(Sender: TObject; Node: TTreeNode);
    procedure vt_MainMenuInitNode(const Sender: TCustomTreeview; const Node: TTreeNode);
    procedure LoadMenuNode ( const ATree : TCustomTreeview ; const AMenuItem : TMenuItem ; const ParentNode : TTreeNode ; const aajouter, aSearchValidity : Boolean ); virtual;
    {$ENDIF}
    procedure vt_MenuIniClick(Sender: TObject);
  private
    gMenuItem : TMenuItem;
    FMenuCustomize : TExtMenuCustomize;
    procedure GetMenu(const AMenuItem: TMenuItem; var AMenuFound: TMenuItem;
      const MenuNameToFind: String);
    {$IFDEF VIRTUALTREES}
    procedure p_ShowMenu(const ATree: TVirtualStringTree; const AMenu: TMenu ; const aSearchValidity : Boolean );
    procedure vt_MenuNodeChange(const AMenuItem: TMenuItem; const Sender: TVirtualStringTree);
    {$ELSE}
    procedure p_ShowMenu(const ATree: TCustomTreeview; const AMenu: TMenu ; const aSearchValidity : Boolean );
    procedure vt_MenuNodeChange(const AMenuItem: TMenuItem; const Sender: TCustomTreeview);
    {$ENDIF}
    { private declarations }
  public
    constructor Create ( AOwner : TComponent ); override;
    procedure DoClose(var CloseAction: TCloseAction); override;
    destructor Destroy; override;
  published
    property MenuCustomize : TExtMenuCustomize read FMenuCustomize  write FMenuCustomize ;
    { public declarations }
  end;

var
  F_CustomizeMenu: TF_CustomizeMenu = nil;

implementation

uses fonctions_components;

{$IFDEF FPC}
{$R *.lfm}
{$ELSE}
{$R *.DFM}
{$ENDIF}


{ TF_CustomizeMenu }

{$IFDEF VIRTUALTREES}
procedure TF_CustomizeMenu.vt_MainMenuInitNode(Sender: TBaseVirtualTree;
  ParentNode, Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
{$ELSE}
procedure TF_CustomizeMenu.vt_MainMenuInitNode(const Sender: TCustomTreeview; const Node: TTreeNode);
{$ENDIF}
var  CustomerRecord : PCustMenuNode;
begin
  {$IFDEF VIRTUALTREES}
  CustomerRecord := Sender.GetNodeData(Node);
  {$ELSE}
  GetMem(CustomerRecord,SizeOf(TMenuNode));
  Node.Data:=CustomerRecord;
  {$ENDIF}
  Initialize(CustomerRecord^);
  if assigned ( gMenuItem ) Then
    Begin
      CustomerRecord^.Name  := gMenuItem.Name ;
      CustomerRecord^.Title := gMenuItem.Caption ;
      CustomerRecord^.ImageIndex:=gMenuItem.ImageIndex;
    end;
end;

{$IFDEF VIRTUALTREES}
procedure TF_CustomizeMenu.LoadMenuNode(const ATree : TVirtualStringTree ; const AMenuItem: TMenuItem; const ParentNode : PVirtualNode ; const aajouter, aSearchValidity : Boolean );
var lnod_ChildNode: PVirtualNode ;
{$ELSE}
procedure TF_CustomizeMenu.LoadMenuNode(const ATree : TCustomTreeview ; const AMenuItem: TMenuItem; const ParentNode : TTreeNode ; const aajouter, aSearchValidity : Boolean );
var lnod_ChildNode: TTreeNode ;
{$ENDIF}
  i : Integer;
begin
  if aajouter
  and ( not aSearchValidity or ( assigned ( AMenuItem.OnClick ) or assigned ( AMenuItem.Action ) or ( AMenuItem.Count > 0 ))) then
    Begin
      gMenuItem := AMenuItem;
      {$IFDEF VIRTUALTREES}
      lnod_ChildNode := ATree.AddChild ( ParentNode );
      ATree.ValidateNode ( Lnod_ChildNode, False );
      {$ELSE}
      lnod_ChildNode := ATree.Items.AddChild ( ParentNode, '' );
      vt_MainMenuInitNode(ATree,lnod_ChildNode);
      vt_MainMenuGetText(lnod_ChildNode);
      {$ENDIF}
    end
   Else
    lnod_ChildNode:=nil;
  for i := 0 to AMenuItem.Count -1  do
    Begin
      LoadMenuNode( ATree, AMenuItem [ i ], lnod_ChildNode, True, aSearchValidity );
    End ;

end;

destructor TF_CustomizeMenu.Destroy;
begin
  inherited Destroy;
  F_CustomizeMenu := nil;
end;

{$IFDEF VIRTUALTREES}
procedure TF_CustomizeMenu.p_ShowMenu ( const ATree : TVirtualStringTree ; const AMenu : TMenu ; const aSearchValidity : Boolean );
{$ELSE}
procedure TF_CustomizeMenu.p_ShowMenu(const ATree: TCustomTreeview; const AMenu: TMenu ; const aSearchValidity : Boolean );
{$ENDIF}
begin
  if assigned ( AMenu ) Then
    with ATree do
      Begin
        BeginUpdate ;
        {$IFNDEF VIRTUALTREES}Items.{$ENDIF}Clear;
        {$IFDEF VIRTUALTREES}
        NodeDataSize := Sizeof(TMenuNode)+1;
        {$ENDIF}
        Images := AMenu.Images;
        EndUpdate;
        BeginUpdate ;
        LoadMenuNode( ATree, AMenu.Items, nil, False, aSearchValidity );
        EndUpdate;
      end;
end;

procedure TF_CustomizeMenu.FormShow(Sender: TObject);
begin
  p_ShowMenu ( vt_MainMenu, MenuCustomize.MainMenu, True );
  p_ShowMenu ( vt_MenuIni , MenuCustomize.MenuIni , False );
end;

procedure TF_CustomizeMenu.FWClose1Click(Sender: TObject);
begin
  Close;
end;

procedure TF_CustomizeMenu.FWDeleteClick(Sender: TObject);
var  CustomerRecord : PCustMenuNode;
     LMenuItem : TMenuItem ;
begin
  with vt_MenuIni do
  if assigned ( {$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF})
   Then
    Begin
      CustomerRecord := {$IFDEF VIRTUALTREES}GetNodeData( FocusedNode ){$ELSE}Selected.Data{$ENDIF};
      LMenuItem := nil;
      FWDelete.Enabled:=False;
      GetMenu( MenuCustomize.MenuIni.Items, LMenuItem, CustomerRecord^.Name );
      if assigned ( LMenuItem ) Then
        LMenuItem.Free;
      vt_MenuIni.{$IFDEF VIRTUALTREES}DeleteNode(FocusedNode{$ELSE}Items.Delete(Selected{$ENDIF});
    End;
end;

procedure TF_CustomizeMenu.FWInsertClick(Sender: TObject);
var  CustomerRecord : PCustMenuNode;
     LMenuItem, LMenuToAdd, lmenuCloned : TMenuItem ;
     lnod_ChildNode : {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
     lnod_Focus : {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
begin
  with vt_MainMenu do
    if  assigned ( {$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF} ) Then
      Begin
        LMenuItem  := nil;
        LMenuToAdd := nil;
        FWInsert.Enabled:=False;
        CustomerRecord := {$IFDEF VIRTUALTREES}GetNodeData( FocusedNode){$ELSE}Selected.Data{$ENDIF};
        GetMenu( MenuCustomize.MainMenu.Items, LMenuToAdd, CustomerRecord^.Name );
        lmenuCloned := fmi_CloneMenuItem ( LMenuToAdd, MenuCustomize.MenuIni );
        if assigned ( vt_MenuIni.{$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF} ) Then
           lnod_Focus := vt_MenuIni.{$IFDEF VIRTUALTREES}RootNode.FirstChild{$ELSE}Items.GetFirstNode{$ENDIF}
         else
           Begin
             MenuCustomize.MenuIni.Items.Add ( lmenuCloned );
             gMenuItem := LMenuToAdd;
             {$IFDEF VIRTUALTREES}
             lnod_ChildNode := vt_MenuIni.AddChild ( nil );
             vt_MenuIni.ValidateNode ( Lnod_ChildNode, False );
             {$ELSE}
             lnod_ChildNode := vt_MenuIni.Items.AddChild ( nil, '' );
             vt_MainMenuInitNode(vt_MenuIni,lnod_ChildNode);
             vt_MainMenuGetText(lnod_ChildNode);
             {$ENDIF}
             Exit;
           end;
        gMenuItem := lmenuCloned;
        CustomerRecord := {$IFDEF VIRTUALTREES}GetNodeData( lnod_Focus ) {$ELSE}lnod_Focus.Data{$ENDIF};
        GetMenu( MenuCustomize.MenuIni.Items, LMenuItem, CustomerRecord^.Name );
        if ( ch_ajouteravant.Checked )
         Then
          Begin
            LMenuItem.Parent.Insert(LMenuItem.Parent.IndexOf(LMenuItem), lmenuCloned );
            Lnod_ChildNode := vt_MenuIni.{$IFDEF VIRTUALTREES}InsertNode(lnod_Focus, amInsertBefore){$ELSE}Items.Insert(lnod_Focus,''){$ENDIF};
          end
         Else
           Begin
             if ( LMenuItem.Parent.IndexOf(LMenuItem) = LMenuItem.Parent.Count - 1 )
              Then LMenuItem.Parent.Add ( lmenuCloned )
              Else LMenuItem.Parent.Insert(LMenuItem.Parent.IndexOf(LMenuItem) + 1, lmenuCloned );
             Lnod_ChildNode := vt_MenuIni.{$IFDEF VIRTUALTREES}InsertNode(lnod_Focus, amInsertAfter){$ELSE}Items.Add(lnod_Focus,''){$ENDIF};
           end;
         {$IFDEF VIRTUALTREES}
         vt_MenuIni.ValidateNode ( Lnod_ChildNode, False );
         {$ELSE}
         vt_MainMenuInitNode(vt_MenuIni,lnod_ChildNode);
         vt_MainMenuGetText(lnod_ChildNode);
         {$ENDIF}
      End;
end;

procedure TF_CustomizeMenu.vt_MainMenuClick(Sender: TObject);
begin
  vt_MenuNodeChange ( MenuCustomize.MainMenu.Items, vt_MainMenu );
end;

procedure TF_CustomizeMenu.FormCreate(Sender: TObject);
begin
  gMenuItem := nil;
end;

procedure TF_CustomizeMenu.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
   closeaction := caFree;
end;

procedure TF_CustomizeMenu.vt_MenuIniClick(Sender: TObject);
begin
  vt_MenuNodeChange ( MenuCustomize.MenuIni.Items, vt_MenuIni);
end;


// procedure TF_CustomizeMenu.GetMenu
// AMenuItem : clicked MenuItem
procedure TF_CustomizeMenu.GetMenu(const AMenuItem: TMenuItem; var AMenuFound : TMenuItem; const MenuNameToFind : String);
var i : Integer;
begin
  with FWDelete do
    Begin
      if AMenuFound <> nil Then
        Exit;
      if MenuNameToFind = AMenuItem.Name Then
        Begin
          AMenuFound := AMenuItem;
          Exit;
        end;
      for i := 0 to AMenuItem.Count -1  do
        GetMenu ( AMenuItem [ i ], AMenuFound, MenuNameToFind );

    end;
end;

procedure TF_CustomizeMenu.vt_MenuNodeChange ( const AMenuItem: TMenuItem; const Sender: {$IFDEF VIRTUALTREES}TVirtualStringTree{$ELSE}TCustomTreeview{$ENDIF});
var  CustomerRecord : PCustMenuNode;
     lmenuItem : TMenuItem;
begin
  with Sender do
  if assigned ( {$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF} ) Then
    Begin
      lmenuItem := nil ;
      if assigned ( AMenuItem ) Then
        Begin
          CustomerRecord := {$IFDEF VIRTUALTREES}GetNodeData( FocusedNode ){$ELSE}Selected.Data{$ENDIF};
          GetMenu ( AMenuItem, lmenuItem, CustomerRecord^.Name );
        end;
      if Sender = vt_MainMenu Then
        Begin
          FWInsert.Enabled := (lmenuItem <> nil ) and {$IFDEF VIRTUALTREES}(FocusedNode^.ChildCount=0){$ELSE}not Assigned(Selected.GetLastSubChild){$ENDIF};
        end
       else
         FWDelete.Enabled := lmenuItem <> nil;
    end;
end;

constructor TF_CustomizeMenu.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {$IFDEF VIRTUALTREES}
  vt_MainMenu:=TVirtualStringTree.create(self);
  vt_MenuIni :=TVirtualStringTree.create(self);
  {$ELSE}
  vt_MainMenu:=TTreeView.create(self);
  vt_MenuIni :=TTreeView.create(self);
  {$ENDIF}
  with vt_MenuIni do
    Begin
      Parent := PanelTrees;
      Left := 0;
      Height := 340;
      Top := 0;
      Width := 240;
      Align := alLeft;
      ParentColor := True;
      TabOrder := 2;
      OnClick := vt_MenuIniClick;
      {$IFDEF VIRTUALTREES}
      Colors.UnfocusedColor := clMedGray;
      DefaultText := 'Node';
      Header.AutoSizeIndex := 0;
      with Header.Columns.add do
        Begin
          Position := 0;
          Text := 'Menu personnalisé';
          Width := 238;
        end;
      Header.DefaultHeight := 22;
      Header.Height := 22;
      Header.Options := [hoAutoResize, hoColumnResize, hoDrag, hoShowSortGlyphs, hoVisible];
      OnGetText := vt_MainMenuGetText;
      OnGetImageIndex := vt_MainMenuGetImageIndex;
      OnInitNode := vt_MainMenuInitNode;
      {$ELSE}
      Caption := 'Menu personnalisé';
      DefaultItemHeight := 22;
      OnGetImageIndex := vt_MainMenuGetImageIndex;
      {$ENDIF}
    End;
  Splitter1:=TSplitter.create ( Self );
  with Splitter1 do
   Begin
    Parent := PanelTrees;
    Left := 240;
    Height := 340;
    Top := 0;
    Width := 5;
   end;
  with vt_MainMenu do
   Begin
      Parent := PanelTrees;
      Left := 245;
      Height := 340;
      Top := 0;
      Width := 355;
      Align := alClient;
      ParentColor := True;
      TabOrder := 0;
      OnClick := vt_MainMenuClick;
      {$IFDEF VIRTUALTREES}
      Colors.UnfocusedColor := clMedGray;
      DefaultText := 'Node';
      Header.AutoSizeIndex := 0;
      with Header.Columns.add do
        Begin
          Position := 0;
          Text := 'Menu complet';
          Width := 353;
        end;
      Header.DefaultHeight := 22;
      Header.Height := 22;
      Header.Options := [hoAutoResize, hoColumnResize, hoDrag, hoShowSortGlyphs, hoVisible];
      OnGetText := vt_MainMenuGetText;
      OnGetImageIndex := vt_MainMenuGetImageIndex;
      OnInitNode := vt_MainMenuInitNode;
      {$ELSE}
      Caption := 'Menu complet';
      DefaultItemHeight := 22;
      OnGetImageIndex := vt_MainMenuGetImageIndex;
      {$ENDIF}
    End;

end;

{$IFDEF VIRTUALTREES}
procedure TF_CustomizeMenu.vt_MainMenuGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
{$ELSE}
  procedure TF_CustomizeMenu.vt_MainMenuGetImageIndex(Sender:TObject; Node : TTreeNode);
{$ENDIF}
var  CustomerRecord : PCustMenuNode;
begin
  CustomerRecord := {$IFDEF VIRTUALTREES}Sender.GetNodeData(Node){$ELSE}Node.Data{$ENDIF};
  {$IFNDEF VIRTUALTREES}Node.{$ENDIF}ImageIndex := CustomerRecord^.ImageIndex;
end;

{$IFDEF VIRTUALTREES}
procedure TF_CustomizeMenu.vt_MainMenuGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: {$IFDEF FPC}String{$ELSE}WideString{$ENDIF});
{$ELSE}
procedure TF_CustomizeMenu.vt_MainMenuGetText(
  Node: TTreeNode);
{$ENDIF}
var  CustomerRecord : PCustMenuNode;
begin
   CustomerRecord := {$IFDEF VIRTUALTREES}Sender.GetNodeData(Node){$ELSE}Node.Data{$ENDIF};
   {$IFDEF VIRTUALTREES}CellText{$ELSE}Node.Text{$ENDIF} := StringReplace(CustomerRecord^.Title,'&','',[rfReplaceAll]);
end;

procedure TF_CustomizeMenu.DoClose(var CloseAction: TCloseAction);
begin
  inherited DoClose(CloseAction);
  CloseAction:=caFree;
end;

{$IFDEF VERSIONS}
initialization
  p_ConcatVersion ( gVer_F_CustomizeMenu );
{$ENDIF}
end.

